@@ -1,10 +1,10 @@
-from typing import List, Optional, Type, Union
+from typing import Any, Optional, Type, Union
 
 from pydantic import BaseModel, field_validator
 from typing_extensions import Annotated, Doc
 
-from esmerald.datastructures.msgspec import Struct
-from esmerald.enums import MediaType
+from esmerald.core.datastructures.msgspec import Struct
+from esmerald.utils.enums import MediaType
 
 
 class OpenAPIResponse(BaseModel):
@@ -35,7 +35,14 @@ class OpenAPIResponse(BaseModel):
     """
 
     model: Annotated[
-        Union[Type[BaseModel], List[Type[BaseModel]], Type[Struct], List[Type[Struct]]],
+        Union[
+            Type[BaseModel],
+            list[Type[BaseModel]],
+            Type[Struct],
+            list[Type[Struct]],
+            Type[Any],
+            list[Type[Any]],
+        ],
         Doc(
             """
             A `pydantic.BaseModel` type of object of a `list` of
@@ -57,7 +64,7 @@ class OpenAPIResponse(BaseModel):
             # Single
             OpenAPIResponse(model=Error)
 
-            # List
+            # list
             OpenAPIResponse(model=[Error])
             ```
             """
@@ -93,8 +100,22 @@ class OpenAPIResponse(BaseModel):
     @field_validator("model", mode="before")
     def validate_model(
         cls,
-        model: Union[Type[BaseModel], List[Type[BaseModel]], Type[Struct], List[Type[Struct]]],
-    ) -> Union[Type[BaseModel], List[Type[BaseModel]], Type[Struct], List[Type[Struct]]]:
+        model: Union[
+            Type[BaseModel],
+            list[Type[BaseModel]],
+            Type[Struct],
+            list[Type[Struct]],
+            Type[Any],
+            list[Type[Any]],
+        ],
+    ) -> Union[
+        Type[BaseModel],
+        list[Type[BaseModel]],
+        Type[Struct],
+        list[Type[Struct]],
+        Type[Any],
+        list[Type[Any]],
+    ]:
         if isinstance(model, list) and len(model) > 1:
             raise ValueError(
                 "The representation of a list of models in OpenAPI can only be a total of one. Example: OpenAPIResponse(model=[MyModel])."
