@@ -2,7 +2,16 @@ import http.client
 import inspect
 import json
 import warnings
-from typing import Any, Dict, List, Optional, Sequence, Set, Tuple, Union, cast
+from typing import (
+    Any,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
+    _GenericAlias,
+    cast,
+)
 
 from lilya._internal._path import clean_path
 from lilya.middleware import DefineMiddleware
@@ -15,7 +24,6 @@ from pydantic.fields import FieldInfo
 from pydantic.json_schema import GenerateJsonSchema, JsonSchemaValue
 from typing_extensions import Literal
 
-from esmerald.enums import MediaType
 from esmerald.openapi.constants import METHODS_WITH_BODY, REF_PREFIX, REF_TEMPLATE
 from esmerald.openapi.models import (
     Contact,
@@ -38,21 +46,64 @@ from esmerald.openapi.utils import (
 )
 from esmerald.params import Param
 from esmerald.routing import gateways, router
+from esmerald.routing.core._internal import (
+    convert_annotation_to_pydantic_model,
+)
+from esmerald.security.oauth2.oauth import SecurityBase
 from esmerald.typing import Undefined
-from esmerald.utils.constants import DATA, PAYLOAD
-from esmerald.utils.helpers import is_class_and_subclass
+from esmerald.utils.dependencies import is_base_requires
+from esmerald.utils.enums import MediaType
+from esmerald.utils.helpers import is_class_and_subclass, is_union
+
+ADDITIONAL_TYPES = ["bool", "list", "dict"]
+TRANSFORMER_TYPES_KEYS = list(TRANSFORMER_TYPES.keys())
+TRANSFORMER_TYPES_KEYS += ADDITIONAL_TYPES
 
 
-def get_flat_params(route: Union[router.HTTPHandler, Any]) -> List[Any]:
-    """Gets all the neded params of the request and route"""
+def get_flat_params(route: Union[router.HTTPHandler, Any], body_fields: list[str]) -> list[Any]:
+    """
+    Gets all the neded params of the request and route.
+    """
     path_params = [param.field_info for param in route.transformer.get_path_params()]
     cookie_params = [param.field_info for param in route.transformer.get_cookie_params()]
     header_params = [param.field_info for param in route.transformer.get_header_params()]
 
+    handler_dependencies = set(route.get_dependencies().keys())
+    body_encoder_fields = route.body_encoder_fields
+    handler_query_params = [
+        param
+        for param in route.transformer.get_query_params()
+        if param.field_alias not in body_encoder_fields
+        and not param.is_security
+        and param.field_alias not in handler_dependencies
+    ]
+
     query_params = []
-    for param in route.transformer.get_query_params():
-        if param.field_info.annotation.__class__.__name__ in TRANSFORMER_TYPES.keys():
-            query_params.append(param.field_info)
+    for param in handler_query_params:
+        is_union_or_optional = is_union(param.field_info.annotation)
+
+        if param.field_info.alias in body_fields:
+            continue
+
+        if param.is_security or param.is_requires_dependency:
+            continue
+
+        # Making sure all the optional and union types are included
+        if is_union_or_optional:
+            if not is_base_requires(param.field_info.default):
+                query_params.append(param.field_info)
+
+        else:
+            if isinstance(param.field_info.annotation, _GenericAlias) and not is_base_requires(
+                param.field_info.default
+            ):
+                query_params.append(param.field_info)
+            elif (
+                param.field_info.annotation.__class__.__name__ in TRANSFORMER_TYPES_KEYS
+                or param.field_info.annotation.__name__ in TRANSFORMER_TYPES_KEYS
+            ):
+                if not is_base_requires(param.field_info.default):
+                    query_params.append(param.field_info)
 
     return path_params + query_params + cookie_params + header_params
 
@@ -68,24 +119,24 @@ def get_openapi_security_schemes(schemes: Any) -> Tuple[dict, list]:
         if inspect.isclass(security_requirement):
             security_requirement = security_requirement()
 
-        if not isinstance(security_requirement, SecurityScheme):
+        if not isinstance(security_requirement, (SecurityScheme, SecurityBase)):
             raise ValueError(
                 "Security schemes must subclass from `esmerald.openapi.models.SecurityScheme`"
             )
+
         security_definition = security_requirement.model_dump(by_alias=True, exclude_none=True)
         security_name = security_requirement.scheme_name
         security_definitions[security_name] = security_definition
         operation_security.append({security_name: security_requirement})
-
     return security_definitions, operation_security
 
 
 def get_fields_from_routes(
-    routes: Sequence[BasePath], request_fields: Optional[List[FieldInfo]] = None
-) -> List[FieldInfo]:
+    routes: Sequence[BasePath], request_fields: Optional[list[FieldInfo]] = None
+) -> list[FieldInfo]:
     """Extracts the fields from the given routes of Esmerald"""
-    body_fields: List[FieldInfo] = []
-    response_from_routes: List[FieldInfo] = []
+    body_fields: list[FieldInfo] = []
+    response_from_routes: list[FieldInfo] = []
 
     if not request_fields:
         request_fields = []
@@ -100,20 +151,16 @@ def get_fields_from_routes(
         ):
             handler = cast(router.HTTPHandler, route.handler)
 
-            # Get the data_field
-            if (
-                DATA in handler.signature_model.model_fields
-                or PAYLOAD in handler.signature_model.model_fields
-            ):
-                data_field = handler.data_field
-                body_fields.append(data_field)
+            if handler.data_field:
+                body_fields.append(handler.data_field)
 
             if handler.response_models:
                 for _, response in handler.response_models.items():
                     response_from_routes.append(response)
 
             # Get the params from the transformer
-            params = get_flat_params(handler)
+            body_fields_names = [field.alias for field in body_fields]
+            params = get_flat_params(handler, body_fields_names)
             if params:
                 request_fields.extend(params)
 
@@ -121,24 +168,29 @@ def get_fields_from_routes(
 
 
 def get_openapi_operation(
-    *, route: Union[router.HTTPHandler, Any], operation_ids: Set[str]
-) -> Dict[str, Any]:  # pragma: no cover
+    *, route: gateways.Gateway, operation_ids: Set[str]
+) -> dict[str, Any]:  # pragma: no cover
     operation = Operation()
-    operation.tags = route.get_handler_tags()
+    operation.tags = route.handler.get_handler_tags()
 
-    if route.summary:
-        operation.summary = route.summary
+    # Handle the routing summary
+    if route.handler.summary:
+        operation.summary = route.handler.summary
     else:
-        operation.summary = route.name.replace("_", " ").replace("-", " ").title()
+        name = route.handler.name or route.name
+        operation.summary = name.replace("_", " ").replace("-", " ").title()
 
-    if route.description:
-        operation.description = route.description
+    # Handle the handler description
+    if route.handler.description:
+        operation.description = route.handler.description
+    else:
+        operation.description = inspect.cleandoc(route.handler.fn.__doc__ or "")
 
-    operation_id = route.operation_id
+    operation_id = getattr(route, "operation_id", None) or route.handler.operation_id
 
     if operation_id in operation_ids:
         message = (
-            f"Duplicate Operation ID {operation_id} for function " + f"{route.handler.__name__}"
+            f"Duplicate Operation ID {operation_id} for function " + f"{route.handler.fn.__name__}"
         )
         file_name = getattr(route.handler, "__globals__", {}).get("__file__")
         if file_name:
@@ -149,6 +201,8 @@ def get_openapi_operation(
     operation.operationId = operation_id
     if route.deprecated:
         operation.deprecated = route.deprecated
+    elif route.handler.deprecated:
+        operation.deprecated = route.handler.deprecated
 
     operation_schema = operation.model_dump(exclude_none=True, by_alias=True)
     return operation_schema
@@ -157,8 +211,8 @@ def get_openapi_operation(
 def get_openapi_operation_parameters(
     *,
     all_route_params: Sequence[FieldInfo],
-    field_mapping: Dict[Tuple[FieldInfo, Literal["validation", "serialization"]], JsonSchemaValue],
-) -> List[Dict[str, Any]]:  # pragma: no cover
+    field_mapping: dict[Tuple[FieldInfo, Literal["validation", "serialization"]], JsonSchemaValue],
+) -> list[dict[str, Any]]:  # pragma: no cover
     parameters = []
     for param in all_route_params:
         field_info = cast(Param, param)
@@ -178,14 +232,12 @@ def get_openapi_operation_parameters(
             required=param.is_required(),
             schema=param_schema,  # type: ignore
         )
-
         if field_info.description:
             parameter.description = field_info.description
         if field_info.examples is not None:
             parameter.example = json.dumps(field_info.examples)
         if field_info.deprecated:
             parameter.deprecated = field_info.deprecated  # type: ignore
-
         parameters.append(parameter.model_dump(by_alias=True, exclude_none=True))
     return parameters
 
@@ -193,8 +245,8 @@ def get_openapi_operation_parameters(
 def get_openapi_operation_request_body(
     *,
     data_field: Optional[FieldInfo],
-    field_mapping: Dict[Tuple[FieldInfo, Literal["validation", "serialization"]], JsonSchemaValue],
-) -> Optional[Dict[str, Any]]:  # pragma: no cover
+    field_mapping: dict[Tuple[FieldInfo, Literal["validation", "serialization"]], JsonSchemaValue],
+) -> Optional[dict[str, Any]]:  # pragma: no cover
     if not data_field:
         return None
 
@@ -202,15 +254,15 @@ def get_openapi_operation_request_body(
     schema = get_schema_from_model_field(field=data_field, field_mapping=field_mapping)
 
     field_info = data_field
-    extra = cast("Dict[str, Any]", data_field.json_schema_extra)
+    extra = cast("dict[str, Any]", data_field.json_schema_extra)
     request_media_type = extra.get("media_type").value
     required = field_info.is_required()
 
-    request_data_oai: Dict[str, Any] = {}
+    request_data_oai: dict[str, Any] = {}
     if required:
         request_data_oai["required"] = required
 
-    request_media_content: Dict[str, Any] = {"schema": schema}
+    request_media_content: dict[str, Any] = {"schema": schema}
     if field_info.examples is not None:
         request_media_content["example"] = json.dumps(field_info.examples)
     request_data_oai["content"] = {request_media_type: request_media_content}
@@ -221,12 +273,12 @@ def get_openapi_path(
     *,
     route: Union[gateways.Gateway, gateways.WebhookGateway],
     operation_ids: Set[str],
-    field_mapping: Dict[Tuple[FieldInfo, Literal["validation", "serialization"]], JsonSchemaValue],
+    field_mapping: dict[Tuple[FieldInfo, Literal["validation", "serialization"]], JsonSchemaValue],
     is_deprecated: bool = False,
-) -> Tuple[Dict[str, Any], Dict[str, Any], Dict[str, Any]]:  # pragma: no cover
-    path: Dict[str, Any] = {}
-    security_schemes: Dict[str, Any] = {}
-    definitions: Dict[str, Any] = {}
+) -> Tuple[dict[str, Any], dict[str, Any], dict[str, Any]]:  # pragma: no cover
+    path: dict[str, Any] = {}
+    security_schemes: dict[str, Any] = {}
+    definitions: dict[str, Any] = {}
 
     assert route.handler.methods is not None, "Methods must be a list"
     route_response_media_type: str = None
@@ -236,9 +288,9 @@ def get_openapi_path(
         internal_response = create_internal_response(handler)
         route_response_media_type = internal_response.media_type
     else:
-        assert (
-            handler.response_class.media_type is not None
-        ), "`media_type` is required in the response class."
+        assert handler.response_class.media_type is not None, (
+            "`media_type` is required in the response class."
+        )
         route_response_media_type = handler.response_class.media_type
 
     # If routes do not want to be included in the schema generation
@@ -247,12 +299,12 @@ def get_openapi_path(
 
     # For each method
     for method in route.handler.methods:
-        operation = get_openapi_operation(route=handler, operation_ids=operation_ids)
+        operation = get_openapi_operation(route=route, operation_ids=operation_ids)  # type: ignore
         # If the parent if marked as deprecated, it takes precedence
         if is_deprecated or route.deprecated:
             operation["deprecated"] = is_deprecated if is_deprecated else route.deprecated
 
-        parameters: List[Dict[str, Any]] = []
+        parameters: list[dict[str, Any]] = []
         security_definitions, operation_security = get_openapi_security_schemes(
             handler.get_security_schemes()
         )
@@ -263,7 +315,12 @@ def get_openapi_path(
         if security_definitions:
             security_schemes.update(security_definitions)
 
-        all_route_params = get_flat_params(handler)
+        body_fields = []
+        if handler.data_field:
+            body_fields.append(handler.data_field)
+
+        body_fields_names = [field.alias for field in body_fields]
+        all_route_params = get_flat_params(handler, body_fields_names)
         operation_parameters = get_openapi_operation_parameters(
             all_route_params=all_route_params,
             field_mapping=field_mapping,
@@ -289,9 +346,9 @@ def get_openapi_path(
                 operation["requestBody"] = request_data_oai
 
         status_code = str(handler.status_code)
-        operation.setdefault("responses", {}).setdefault(status_code, {})[
-            "description"
-        ] = handler.response_description
+        operation.setdefault("responses", {}).setdefault(status_code, {})["description"] = (
+            handler.response_description
+        )
 
         # Media type
         if route_response_media_type and is_status_code_allowed(handler.status_code):
@@ -316,7 +373,7 @@ def get_openapi_path(
                 openapi_response = operation_responses.setdefault(status_code_key, {})
 
                 field = handler.response_models.get(additional_status_code)
-                additional_field_schema: Optional[Dict[str, Any]] = None
+                additional_field_schema: Optional[dict[str, Any]] = None
                 model_schema = process_response.model_json_schema()
 
                 if field:
@@ -346,6 +403,22 @@ def get_openapi_path(
                 dict_update(openapi_response, model_schema)
                 openapi_response["description"] = description
 
+        # Convert to automatic response detection if none is provided by the
+        # responses of the handler.
+        if handler.handler_signature.return_annotation:
+            response_schema = convert_annotation_to_pydantic_model(
+                handler.handler_signature.return_annotation
+            )
+
+            if (
+                hasattr(response_schema, "model_json_schema")
+                and status_code not in handler.responses
+                and int(status_code) not in handler.responses
+            ):
+                operation["responses"][status_code]["content"][route_response_media_type][
+                    "schema"
+                ] = response_schema.model_json_schema()
+
         http422 = str(HTTP_422_UNPROCESSABLE_ENTITY)
         if (all_route_params or handler.data_field) and not any(
             status in operation["responses"] for status in {http422, "4XX", "default"}
@@ -417,13 +490,13 @@ def get_openapi(
     summary: Optional[str] = None,
     description: Optional[str] = None,
     routes: Sequence[BasePath],
-    tags: Optional[List[str]] = None,
-    servers: Optional[List[Dict[str, Union[str, Any]]]] = None,
+    tags: Optional[list[str]] = None,
+    servers: Optional[list[dict[str, Union[str, Any]]]] = None,
     terms_of_service: Optional[Union[str, AnyUrl]] = None,
     contact: Optional[Contact] = None,
     license: Optional[License] = None,
     webhooks: Optional[Sequence[BasePath]] = None,
-) -> Dict[str, Any]:  # pragma: no cover
+) -> dict[str, Any]:  # pragma: no cover
     """
     Builds the whole OpenAPI route structure and object
     """
@@ -441,7 +514,7 @@ def get_openapi(
     if license:
         info.license = license
 
-    output: Dict[str, Any] = {
+    output: dict[str, Any] = {
         "openapi": openapi_version,
         "info": info.model_dump(exclude_none=True, by_alias=True),
     }
@@ -449,9 +522,9 @@ def get_openapi(
     if servers:
         output["servers"] = servers
 
-    components: Dict[str, Dict[str, Any]] = {}
-    paths: Dict[str, Dict[str, Any]] = {}
-    webhooks_paths: Dict[str, Dict[str, Any]] = {}
+    components: dict[str, dict[str, Any]] = {}
+    paths: dict[str, dict[str, Any]] = {}
+    webhooks_paths: dict[str, dict[str, Any]] = {}
     operation_ids: Set[str] = set()
     all_fields = get_fields_from_routes(list(routes or []) + list(webhooks or []))
     schema_generator = GenerateJsonSchema(ref_template=REF_TEMPLATE)
@@ -469,7 +542,7 @@ def get_openapi(
         prefix: Optional[str] = "",
         is_webhook: bool = False,
         is_deprecated: bool = False,
-    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+    ) -> Tuple[dict[str, Any], dict[str, Any]]:
         for route in routes:
             if app.router.deprecated:
                 is_deprecated = True
@@ -553,4 +626,4 @@ def get_openapi(
 
     openapi = OpenAPI(**output)
     model_dump = openapi.model_dump_json(by_alias=True, exclude_none=True)
-    return cast(Dict[str, Any], loads(model_dump))
+    return cast(dict[str, Any], loads(model_dump))
