@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Optional
+from typing import Any, Optional
 
 from pydantic import BaseModel
 
@@ -8,7 +8,7 @@ from esmerald.openapi.constants import REF_PREFIX
 class BaseDetail(BaseModel):
     title: str
     type: str
-    items: Dict[str, Any]
+    items: dict[str, Any]
 
 
 class Detail(BaseDetail): ...
@@ -19,8 +19,8 @@ class Loc(BaseDetail): ...
 
 class Property(BaseModel):
     loc: Optional[Loc] = None
-    msg: Optional[Dict[str, str]] = None
-    type: Optional[Dict[str, str]] = None
+    msg: Optional[dict[str, str]] = None
+    type: Optional[dict[str, str]] = None
     detail: Optional[Detail] = None
 
 
@@ -28,7 +28,7 @@ class ValidationErrorModel(BaseModel):
     title: str
     type: str
     properties: Property
-    required: Optional[List[str]] = None
+    required: Optional[list[str]] = None
 
 
 validation_error_definition = ValidationErrorModel(
