@@ -1,12 +1,14 @@
-# from dataclasses import dataclass
-from typing import Any, Callable, Dict, List, Optional, Union
+import inspect
+from functools import cached_property
+from typing import Any, Callable, Optional, Sequence, Union
 
-from pydantic.dataclasses import dataclass
 from pydantic.fields import AliasChoices, AliasPath, FieldInfo
 
-from esmerald.enums import EncodingType, ParamType
+from esmerald.security.scopes import Scopes
 from esmerald.typing import Undefined
 from esmerald.utils.constants import IS_DEPENDENCY, SKIP_VALIDATION
+from esmerald.utils.enums import EncodingType, ParamType
+from esmerald.utils.helpers import is_class_and_subclass, make_callable
 
 _PyUndefined: Any = Undefined
 
@@ -43,7 +45,7 @@ class Param(FieldInfo):
         max_digits: Optional[int] = _PyUndefined,
         strict: Optional[bool] = _PyUndefined,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         deprecated: Optional[bool] = None,
         include_in_schema: bool = True,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
@@ -52,13 +54,14 @@ class Param(FieldInfo):
         validate_default: bool = True,
         init_var: bool = True,
         kw_only: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
         self.deprecated = deprecated
         self.examples = examples
         self.include_in_schema = include_in_schema
         self.allow_none = allow_none
 
-        extra: Dict[str, Any] = {}
+        extra: dict[str, Any] = {}
         extra.update(header=header)
         extra.update(cookie=cookie)
         extra.update(query=query)
@@ -71,6 +74,9 @@ class Param(FieldInfo):
         extra.update(include_in_schema=self.include_in_schema)
         extra.update(allow_none=self.allow_none)
 
+        if json_schema_extra is not None:
+            extra.update(json_schema_extra)
+
         super().__init__(
             annotation=annotation,
             default=default,
@@ -133,7 +139,7 @@ class Header(Param):
         max_digits: Optional[int] = _PyUndefined,
         strict: Optional[bool] = _PyUndefined,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         deprecated: Optional[bool] = None,
         include_in_schema: bool = True,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
@@ -142,6 +148,7 @@ class Header(Param):
         validate_default: bool = True,
         init_var: bool = True,
         kw_only: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
         super().__init__(
             default=default,
@@ -176,6 +183,7 @@ class Header(Param):
             frozen=frozen,
             init_var=init_var,
             kw_only=kw_only,
+            json_schema_extra=json_schema_extra,
         )
 
 
@@ -208,7 +216,7 @@ class Cookie(Param):
         max_digits: Optional[int] = _PyUndefined,
         strict: Optional[bool] = _PyUndefined,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         deprecated: Optional[bool] = None,
         include_in_schema: bool = True,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
@@ -217,6 +225,7 @@ class Cookie(Param):
         validate_default: bool = True,
         init_var: bool = True,
         kw_only: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
         super().__init__(
             default=default,
@@ -251,6 +260,7 @@ class Cookie(Param):
             frozen=frozen,
             init_var=init_var,
             kw_only=kw_only,
+            json_schema_extra=json_schema_extra,
         )
 
 
@@ -283,7 +293,7 @@ class Query(Param):
         max_digits: Optional[int] = _PyUndefined,
         strict: Optional[bool] = _PyUndefined,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         deprecated: Optional[bool] = None,
         include_in_schema: bool = True,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
@@ -292,6 +302,7 @@ class Query(Param):
         validate_default: bool = True,
         init_var: bool = True,
         kw_only: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
         super().__init__(
             default=default,
@@ -326,6 +337,7 @@ class Query(Param):
             frozen=frozen,
             init_var=init_var,
             kw_only=kw_only,
+            json_schema_extra=json_schema_extra,
         )
 
 
@@ -353,7 +365,7 @@ class Path(Param):
         max_digits: Optional[int] = _PyUndefined,
         strict: Optional[bool] = _PyUndefined,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         deprecated: Optional[bool] = None,
         include_in_schema: bool = True,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
@@ -362,6 +374,7 @@ class Path(Param):
         validate_default: bool = True,
         init_var: bool = True,
         kw_only: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
         super().__init__(
             default=default,
@@ -391,6 +404,7 @@ class Path(Param):
             frozen=frozen,
             init_var=init_var,
             kw_only=kw_only,
+            json_schema_extra=json_schema_extra,
         )
 
 
@@ -420,7 +434,7 @@ class Body(FieldInfo):
         min_length: Optional[int] = None,
         max_length: Optional[int] = None,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
         discriminator: Optional[str] = None,
         frozen: Optional[bool] = None,
@@ -428,8 +442,9 @@ class Body(FieldInfo):
         init_var: bool = True,
         kw_only: bool = True,
         include_in_schema: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
-        extra: Dict[str, Any] = {}
+        extra: dict[str, Any] = {}
         self.media_type = media_type
         self.content_encoding = content_encoding
         self.examples = examples
@@ -441,6 +456,8 @@ class Body(FieldInfo):
         extra.update(embed=embed)
         extra.update(allow_none=allow_none)
 
+        json_schema_extra = extra if json_schema_extra is None else json_schema_extra.update(extra)
+
         super().__init__(
             default=default,
             default_factory=default_factory,
@@ -459,7 +476,7 @@ class Body(FieldInfo):
             min_length=min_length,
             max_length=max_length,
             pattern=pattern,
-            json_schema_extra=extra,
+            json_schema_extra=json_schema_extra,
             validate_default=validate_default,
             validation_alias=validation_alias,
             discriminator=discriminator,
@@ -494,7 +511,7 @@ class Form(Body):
         min_length: Optional[int] = None,
         max_length: Optional[int] = None,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
         discriminator: Optional[str] = None,
         max_digits: Optional[int] = _PyUndefined,
@@ -504,6 +521,7 @@ class Form(Body):
         init_var: bool = True,
         kw_only: bool = True,
         include_in_schema: bool = True,
+        json_schema_extra: Optional[dict[str, Any]] = None,
     ) -> None:
         super().__init__(
             default=default,
@@ -534,6 +552,7 @@ class Form(Body):
             include_in_schema=include_in_schema,
             max_digits=max_digits,
             strict=strict,
+            json_schema_extra=json_schema_extra,
         )
 
 
@@ -558,7 +577,7 @@ class File(Form):
         min_length: Optional[int] = None,
         max_length: Optional[int] = None,
         pattern: Optional[str] = None,
-        examples: Optional[List[Any]] = None,
+        examples: Optional[list[Any]] = None,
         validation_alias: Optional[Union[str, AliasPath, AliasChoices]] = None,
         discriminator: Optional[str] = None,
         max_digits: Optional[int] = _PyUndefined,
@@ -621,7 +640,7 @@ class Injects(FieldInfo):
         allow_none: bool = True,
     ) -> None:
         self.allow_none = allow_none
-        self.extra: Dict[str, Any] = {
+        self.extra: dict[str, Any] = {
             IS_DEPENDENCY: True,
             SKIP_VALIDATION: skip_validation,
             "allow_none": self.allow_none,
@@ -629,25 +648,83 @@ class Injects(FieldInfo):
         super().__init__(default=default, json_schema_extra=self.extra)
 
 
-@dataclass
-class DirectInject:  # pragma: no cover
+class BaseRequires:
+    """
+    A class that represents a requirement with an optional dependency and caching behavior.
+
+    This object serves as a base class for other classes that require dependencies.
+
+    Attributes:
+        dependency (Optional[Callable[..., Any]]): An optional callable that represents the dependency.
+        use_cache (bool): A flag indicating whether to use caching for the dependency. Defaults to True.
+
+    Methods:
+        __repr__(): Returns a string representation of the Requires instance.
+    """
+
+    def __init__(self, dependency: Optional[Callable[..., Any]] = None, *, use_cache: bool = True):
+        """
+        Initializes a Requires instance.
+
+        Args:
+            dependency (Optional[Callable[..., Any]]): An optional callable that represents the dependency.
+            use_cache (bool): A flag indicating whether to use caching for the dependency. Defaults to True.
+        """
+
+        """
+        Returns a string representation of the Requires instance.
+
+        Returns:
+            str: A string representation of the Requires instance.
+        """
+        self.dependency = dependency
+        self.use_cache = use_cache
+
+        if not callable(dependency):
+            dependency = make_callable(dependency)
+        self.signature_model = inspect.signature(dependency) if dependency else None
+
+    def __repr__(self) -> str:
+        attr = getattr(self.dependency, "__name__", type(self.dependency).__name__)
+        cache = "" if self.use_cache else ", use_cache=False"
+        return f"{self.__class__.__name__}({attr}{cache})"
+
+
+class Requires(BaseRequires): ...
+
+
+class Security(BaseRequires):
+    """
+    A class used to represent security requirements for a particular operation.
+
+    Attributes:
+    ----------
+    dependency : Optional[Callable[..., Any]]
+        A callable that represents the dependency required for security.
+    scopes : Optional[Sequence[str]]
+        A sequence of scopes required for the security. Defaults to an empty list.
+    use_cache : bool
+        A flag indicating whether to use cache. Defaults to True.
+
+    Methods:
+    -------
+    __init__(self, dependency: Optional[Callable[..., Any]] = None, *, scopes: Optional[Sequence[str]] = None, use_cache: bool = True)
+        Initializes the Security class with the given dependency, scopes, and use_cache flag.
+    """
+
     def __init__(
         self,
         dependency: Optional[Callable[..., Any]] = None,
         *,
+        scopes: Optional[Sequence[str]] = None,
         use_cache: bool = True,
-        allow_none: bool = True,
-    ) -> None:
-        self.dependency = dependency
-        self.use_cache = use_cache
-        self.allow_none = allow_none
+    ):
+        super().__init__(dependency=dependency, use_cache=use_cache)
+        self.scopes = scopes or []
 
-    def __hash__(self) -> int:
-        values: Dict[str, Any] = {}
-        for key, value in self.__dict__.items():
-            values[key] = None
-            if isinstance(value, (list, set)):
-                values[key] = tuple(value)
-            else:
-                values[key] = value
-        return hash((type(self),) + tuple(values))
+    @cached_property
+    def is_security_scope_dependency(self) -> bool:
+        parameters: dict[str, inspect.Parameter] = dict(self.signature_model.parameters.items())
+        return any(
+            is_class_and_subclass(param.annotation, Scopes) for param in parameters.values()
+        )
