@@ -1,4 +1,4 @@
-from abc import ABC, abstractmethod
+from abc import abstractmethod
 from typing import TYPE_CHECKING, Any
 
 from typing_extensions import Annotated, Doc
@@ -191,7 +191,7 @@ class BasePermission(metaclass=BasePermissionMetaclass):
         return True
 
 
-class BaseAbstractUserPermission(ABC):
+class BaseAbstractUserPermission(BasePermission):
     """
     Abstract Base class for user validation permissions.
     """
@@ -212,7 +212,7 @@ class BaseAbstractUserPermission(ABC):
         This method must be overridden by subclasses.
 
         Args:
-            request: A Starlette 'HTTPConnection' instance.
+            request: A Lilya 'Connection' instance.
 
         Returns:
             bool: True or False
@@ -225,7 +225,7 @@ class BaseAbstractUserPermission(ABC):
         This method must be overridden by subclasses.
 
         Args:
-            request: A Starlette 'HTTPConnection' instance.
+            request: A Lilya 'Connection' instance.
 
         Returns:
             bool: True or False
@@ -278,7 +278,7 @@ class IsAuthenticated(BaseAbstractUserPermission):
     ) -> bool:
         """
         Args:
-            request: A Starlette 'HTTPConnection' instance.
+            request: A Lilya 'Connection' instance.
             apiview: A Esmerald 'APIController' instance or a `APIGateHandler` instance.
 
         Returns:
@@ -300,7 +300,7 @@ class IsAdminUser(BaseAbstractUserPermission):
     ) -> bool:
         """
         Args:
-            request: A Starlette 'HTTPConnection' instance.
+            request: A Lilya 'Connection' instance.
             apiview: A Esmerald 'APIController' instance or a `APIGateHandler` instance.
 
         Returns:
@@ -322,7 +322,7 @@ class IsAuthenticatedOrReadOnly(BaseAbstractUserPermission):
     ) -> bool:
         """
         Args:
-            request: A Starlette 'HTTPConnection' instance.
+            request: A Lilya 'Connection' instance.
             apiview: A Esmerald 'APIController' instance or a `APIGateHandler` instance.
 
         Returns:
