@@ -1,6 +1,7 @@
 import re
-from typing import TYPE_CHECKING, Any, Callable, List, Optional, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, Callable, Optional, Sequence, Type, Union, cast
 
+from lilya._internal._connection import Connection  # noqa
 from lilya._internal._path import clean_path
 from lilya._utils import is_class_and_subclass
 from lilya.middleware import DefineMiddleware
@@ -8,15 +9,14 @@ from lilya.routing import Path as LilyaPath, WebSocketPath as LilyaWebSocketPath
 from lilya.types import Receive, Scope, Send
 from typing_extensions import Annotated, Doc
 
+from esmerald.permissions.utils import is_esmerald_permission, is_lilya_permission, wrap_permission
 from esmerald.routing.apis.base import View
-from esmerald.routing.base import BaseInterceptorMixin
-from esmerald.typing import Void, VoidType
+from esmerald.routing.core.base import Dispatcher
 from esmerald.utils.helpers import clean_string
 
 if TYPE_CHECKING:  # pragma: no cover
-    from openapi_schemas_pydantic.v3_1_0.security_scheme import SecurityScheme
-
-    from esmerald.interceptors.types import Interceptor
+    from esmerald.core.interceptors.types import Interceptor
+    from esmerald.openapi.schemas.v3_1_0.security_scheme import SecurityScheme
     from esmerald.permissions.types import Permission
     from esmerald.routing.router import HTTPHandler, WebhookHandler, WebSocketHandler
     from esmerald.types import Dependencies, ExceptionHandlerMap, Middleware, ParentType
@@ -24,12 +24,12 @@ if TYPE_CHECKING:  # pragma: no cover
 
 class BaseMiddleware:
     def handle_middleware(
-        self, handler: Any, base_middleware: List["Middleware"]
-    ) -> List["Middleware"]:
+        self, handler: Any, base_middleware: list["Middleware"]
+    ) -> list["Middleware"]:
         """
         Handles both types of middlewares for Gateway and WebSocketGateway
         """
-        _middleware: List["Middleware"] = []
+        _middleware: list["Middleware"] = []
 
         if not is_class_and_subclass(handler, View) and not isinstance(handler, View):
             base_middleware += handler.middleware or []
@@ -43,7 +43,6 @@ class BaseMiddleware:
 
 
 class GatewayUtil:
-
     def is_class_based(
         self, handler: Union["HTTPHandler", "WebSocketHandler", "ParentType"]
     ) -> bool:
@@ -61,9 +60,7 @@ class GatewayUtil:
         We need to be able to handle with edge cases when a view does not default a path like `/format` and a default name needs to be passed when its a class based view.
         """
         if self.is_class_based(handler.parent):
-            operation_id = (
-                handler.parent.__class__.__name__.lower() + f"_{name}" + handler.path_format
-            )
+            operation_id = handler.parent.__class__.__name__.lower() + handler.path_format
         else:
             operation_id = name + handler.path_format
 
@@ -75,7 +72,7 @@ class GatewayUtil:
         return operation_id
 
 
-class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
+class Gateway(LilyaPath, Dispatcher, BaseMiddleware, GatewayUtil):
     """
     `Gateway` object class used by Esmerald routes.
 
@@ -112,6 +109,9 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
         "permissions",
         "deprecated",
         "tags",
+        "operation_id",
+        "before_request",
+        "after_request",
     )
 
     def __init__(
@@ -140,7 +140,7 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
         ] = None,
         *,
         handler: Annotated[
-            Union["HTTPHandler", View],
+            Union["HTTPHandler", View, Type[View], Type["HTTPHandler"]],
             Doc(
                 """
             An instance of [handler](https://esmerald.dev/routing/handlers/#http-handlers).
@@ -182,7 +182,7 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
             ),
         ] = None,
         middleware: Annotated[
-            Optional[List["Middleware"]],
+            Optional[list["Middleware"]],
             Doc(
                 """
                 A list of middleware to run for every request. The middlewares of a Gateway will be checked from top-down or [Lilya Middleware](https://www.lilya.dev/middleware/) as they are both converted internally. Read more about [Python Protocols](https://peps.python.org/pep-0544/).
@@ -213,6 +213,73 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
                 """
             ),
         ] = None,
+        before_request: Annotated[
+            Union[Sequence[Callable[[], Any]], None],
+            Doc(
+                """
+                A `list` of events that are trigger after the application
+                processes the request.
+
+                Read more about the [events](https://lilya.dev/lifespan/).
+
+                **Example**
+
+                ```python
+                from edgy import Database, Registry
+
+                from esmerald import Esmerald, Request, Gateway, get
+
+                database = Database("postgresql+asyncpg://user:password@host:port/database")
+                registry = Registry(database=database)
+
+                async def create_user(request: Request):
+                    # Logic to create the user
+                    data = await request.json()
+                    ...
+
+
+                app = Esmerald(
+                    routes=[Gateway("/create", handler=create_user)],
+                    after_request=[database.disconnect],
+                )
+                ```
+                """
+            ),
+        ] = None,
+        after_request: Annotated[
+            Union[Sequence[Callable[[], Any]], None],
+            Doc(
+                """
+                A `list` of events that are trigger after the application
+                processes the request.
+
+                Read more about the [events](https://lilya.dev/lifespan/).
+
+                **Example**
+
+                ```python
+                from edgy import Database, Registry
+
+                from esmerald import Esmerald, Request, Gateway, get
+
+                database = Database("postgresql+asyncpg://user:password@host:port/database")
+                registry = Registry(database=database)
+
+
+                async def create_user(request: Request):
+                    # Logic to create the user
+                    data = await request.json()
+                    ...
+
+
+                app = Esmerald(
+                    routes=[Gateway("/create", handler=create_user)],
+                    after_request=[database.disconnect],
+                )
+                ```
+                """
+            ),
+        ] = None,
         deprecated: Annotated[
             Optional[bool],
             Doc(
@@ -261,11 +328,21 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
                 """
             ),
         ] = None,
+        operation_id: Annotated[
+            Optional[str],
+            Doc(
+                """
+                Unique operation id that allows distinguishing the same handler in different Gateways.
+
+                Used for OpenAPI purposes.
+                """
+            ),
+        ] = None,
     ) -> None:
         if not path:
             path = "/"
         if is_class_and_subclass(handler, View):
-            handler = handler(parent=self)  # type: ignore
+            handler = handler(parent=self)
 
         if not is_from_router:
             self.path = clean_path(path + handler.path)
@@ -276,15 +353,30 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
 
         if not name:
             if not isinstance(handler, View):
-                name = clean_string(handler.fn.__name__)
+                name = handler.name or clean_string(handler.fn.__name__)  # type: ignore
             else:
                 name = clean_string(handler.__class__.__name__)
 
+        else:
+            route_name_list = [name]
+            if not isinstance(handler, View) and handler.name:  # type: ignore
+                route_name_list.append(handler.name)  # type: ignore
+                name = ":".join(route_name_list)
+
         # Handle middleware
         self.middleware = middleware or []
-        self._middleware: List["Middleware"] = self.handle_middleware(
+        self._middleware: list["Middleware"] = self.handle_middleware(
             handler=handler, base_middleware=self.middleware
         )
+
+        self.__base_permissions__ = permissions or []
+
+        self.__lilya_permissions__ = [
+            wrap_permission(permission)
+            for permission in self.__base_permissions__ or []
+            if is_lilya_permission(permission)
+        ]
+
         super().__init__(
             path=self.path,
             handler=cast(Callable, handler),
@@ -293,18 +385,81 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
             methods=self.methods,
             middleware=self._middleware,
             exception_handlers=exception_handlers,
+            permissions=self.__lilya_permissions__,  # type: ignore
         )
         """
         A "bridge" to a handler and router mapping functionality.
         Since the default Lilya Route handler does not understand the Esmerald handlers,
         the Gateway bridges both functionalities and adds an extra "flair" to be compliant with both class based views and decorated function views.
         """
-        self._interceptors: Union[List["Interceptor"], "VoidType"] = Void
+        self.before_request = before_request if before_request is not None else []
+        self.after_request = after_request if after_request is not None else []
+
+        if self.before_request:
+            if handler.before_request is None:
+                handler.before_request = []
+
+            for before in self.before_request:
+                handler.before_request.insert(0, before)
+
+        if self.after_request:
+            if handler.after_request is None:
+                handler.after_request = []
+
+            for after in self.after_request:
+                handler.after_request.append(after)
+
         self.name = name
         self.handler = cast("Callable", handler)
         self.dependencies = dependencies or {}
         self.interceptors: Sequence["Interceptor"] = interceptors or []
-        self.permissions: Sequence["Permission"] = permissions or []  # type: ignore
+
+        if self.interceptors:
+            if not handler.interceptors:
+                handler.interceptors = self.interceptors
+            else:
+                for interceptor in self.interceptors:
+                    handler.interceptors.insert(0, interceptor)
+
+        # Filter out the lilya unique permissions
+        if self.__lilya_permissions__:
+            self.lilya_permissions: Any = dict(enumerate(self.__lilya_permissions__))
+            if not self.handler.lilya_permissions:
+                self.handler.lilya_permissions = self.lilya_permissions
+            else:
+                handler_lilya_permissions = {
+                    index + len(self.lilya_permissions): permission
+                    for index, permission in enumerate(self.lilya_permissions.values())
+                }
+                self.handler.lilya_permissions = {
+                    **self.lilya_permissions,
+                    **handler_lilya_permissions,
+                }
+        else:
+            self.lilya_permissions = {}
+
+        # Filter out the esmerald unique permissions
+        if self.__base_permissions__:
+            self.permissions: Any = {
+                index: wrap_permission(permission)
+                for index, permission in enumerate(permissions)
+                if is_esmerald_permission(permission)
+            }
+
+            if not self.handler.permissions:
+                self.handler.permissions = self.permissions
+            else:
+                handler_permissions = {
+                    index + len(self.permissions): permission
+                    for index, permission in enumerate(self.permissions.values())
+                }
+                self.handler.permissions = {
+                    **self.permissions,
+                    **handler_permissions,
+                }
+        else:
+            self.permissions = {}
+
         self.response_class = None
         self.response_cookies = None
         self.response_headers = None
@@ -315,13 +470,20 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
         (handler.path_regex, handler.path_format, handler.param_convertors, _) = compile_path(
             self.path
         )
+        self.operation_id = operation_id
 
         if self.is_handler(self.handler):  # type: ignore
-            self.handler.name = self.name
+            if self.operation_id or handler.operation_id is not None:
+                handler_id = self.generate_operation_id(
+                    name=self.name or "",
+                    handler=self.handler,  # type: ignore
+                )
+                self.operation_id = f"{operation_id}_{handler_id}" if operation_id else handler_id
 
-            if not handler.operation_id:
+            elif not handler.operation_id:
                 handler.operation_id = self.generate_operation_id(
-                    name=self.name, handler=self.handler  # type: ignore
+                    name=self.name or "",
+                    handler=self.handler,  # type: ignore
                 )
 
     async def handle_dispatch(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
@@ -331,7 +493,7 @@ class Gateway(LilyaPath, BaseInterceptorMixin, BaseMiddleware, GatewayUtil):
         await self.app(scope, receive, send)
 
 
-class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware):
+class WebSocketGateway(LilyaWebSocketPath, Dispatcher, BaseMiddleware):
     """
     `WebSocketGateway` object class used by Esmerald routes.
 
@@ -371,6 +533,8 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
         "parent",
         "security",
         "tags",
+        "before_request",
+        "after_request",
     )
 
     def __init__(
@@ -399,7 +563,7 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
         ] = None,
         *,
         handler: Annotated[
-            Union["WebSocketHandler", View],
+            Union["WebSocketHandler", View, Type[View], Type["WebSocketHandler"]],
             Doc(
                 """
             An instance of [handler](https://esmerald.dev/routing/handlers/#websocket-handler).
@@ -433,7 +597,7 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
             ),
         ] = None,
         middleware: Annotated[
-            Optional[List["Middleware"]],
+            Optional[list["Middleware"]],
             Doc(
                 """
                 A list of middleware to run for every request. The middlewares of a Gateway will be checked from top-down or [Lilya Middleware](https://www.lilya.dev/middleware/) as they are both converted internally. Read more about [Python Protocols](https://peps.python.org/pep-0544/).
@@ -464,6 +628,28 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
                 """
             ),
         ] = None,
+        before_request: Annotated[
+            Union[Sequence[Callable[[], Any]], None],
+            Doc(
+                """
+                A `list` of events that are trigger after the application
+                processes the request.
+
+                Read more about the [events](https://lilya.dev/lifespan/).
+                """
+            ),
+        ] = None,
+        after_request: Annotated[
+            Union[Sequence[Callable[[], Any]], None],
+            Doc(
+                """
+                A `list` of events that are trigger after the application
+                processes the request.
+
+                Read more about the [events](https://lilya.dev/lifespan/).
+                """
+            ),
+        ] = None,
         is_from_router: Annotated[
             bool,
             Doc(
@@ -477,8 +663,7 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
         if not path:
             path = "/"
         if is_class_and_subclass(handler, View):
-            handler = handler(parent=self)  # type: ignore
-
+            handler = handler(parent=self)
         if not is_from_router:
             self.path = clean_path(path + handler.path)
         else:
@@ -486,34 +671,111 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
 
         if not name:
             if not isinstance(handler, View):
-                name = clean_string(handler.fn.__name__)
+                name = handler.name or clean_string(handler.fn.__name__)  # type: ignore
             else:
                 name = clean_string(handler.__class__.__name__)
 
+        else:
+            route_name_list = [name]
+            if not isinstance(handler, View) and handler.name:  # type: ignore
+                route_name_list.append(handler.name)  # type: ignore
+                name = ":".join(route_name_list)
+
         # Handle middleware
         self.middleware = middleware or []
-        self._middleware: List["Middleware"] = self.handle_middleware(
+        self._middleware: list["Middleware"] = self.handle_middleware(
             handler=handler, base_middleware=self.middleware
         )
         self.is_middleware: bool = False
 
+        self.__base_permissions__ = permissions or []
+        self.__lilya_permissions__ = [
+            wrap_permission(permission)
+            for permission in self.__base_permissions__ or []
+            if is_lilya_permission(permission)
+        ]
         super().__init__(
             path=self.path,
             handler=cast("Callable", handler),
             name=name,
             middleware=self._middleware,
             exception_handlers=exception_handlers,
+            permissions=self.__lilya_permissions__,  # type: ignore
+            before_request=before_request,
+            after_request=after_request,
         )
         """
         A "bridge" to a handler and router mapping functionality.
         Since the default Lilya Route handler does not understand the Esmerald handlers,
         the Gateway bridges both functionalities and adds an extra "flair" to be compliant with both class based views and decorated function views.
         """
-        self._interceptors: Union[List["Interceptor"], "VoidType"] = Void
+        self.before_request = before_request if before_request is not None else []
+        self.after_request = after_request if after_request is not None else []
+
+        if self.before_request:
+            if handler.before_request is None:
+                handler.before_request = []
+
+            for before in self.before_request:
+                handler.before_request.insert(0, before)
+
+        if self.after_request:
+            if handler.after_request is None:
+                handler.after_request = []
+
+            for after in self.after_request:
+                handler.after_request.append(after)
+
         self.handler = cast("Callable", handler)
         self.dependencies = dependencies or {}
         self.interceptors = interceptors or []
-        self.permissions = permissions or []  # type: ignore
+
+        if self.interceptors:
+            if not handler.interceptors:
+                handler.interceptors = self.interceptors
+            else:
+                for interceptor in self.interceptors:
+                    handler.interceptors.insert(0, interceptor)
+
+        # Filter out the lilya unique permissions
+        if self.__lilya_permissions__:
+            self.lilya_permissions: Any = dict(enumerate(self.__lilya_permissions__))
+            if not self.handler.lilya_permissions:
+                self.handler.lilya_permissions = self.lilya_permissions
+            else:
+                handler_lilya_permissions = {
+                    index + len(self.lilya_permissions): permission
+                    for index, permission in enumerate(self.lilya_permissions.values())
+                }
+                self.handler.lilya_permissions = {
+                    **self.lilya_permissions,
+                    **handler_lilya_permissions,
+                }
+        else:
+            self.lilya_permissions = {}
+
+        # Filter out the esmerald unique permissions
+        if self.__base_permissions__:
+            self.permissions: Any = {
+                index: wrap_permission(permission)
+                for index, permission in enumerate(permissions)
+                if is_esmerald_permission(permission)
+            }
+
+            if not self.handler.permissions:
+                self.handler.permissions = self.permissions
+            else:
+                handler_permissions = {
+                    index + len(self.permissions): permission
+                    for index, permission in enumerate(self.permissions.values())
+                }
+                self.handler.permissions = {
+                    **self.permissions,
+                    **handler_permissions,
+                }
+        else:
+            self.permissions = {}
+
         self.include_in_schema = False
         self.parent = parent
         (handler.path_regex, handler.path_format, handler.param_convertors, _) = compile_path(
@@ -529,7 +791,7 @@ class WebSocketGateway(LilyaWebSocketPath, BaseInterceptorMixin, BaseMiddleware)
         await self.app(scope, receive, send)
 
 
-class WebhookGateway(LilyaPath, BaseInterceptorMixin, GatewayUtil):
+class WebhookGateway(LilyaPath, Dispatcher, GatewayUtil):
     """
     `WebhookGateway` object class used by Esmerald routes.
 
@@ -557,13 +819,15 @@ class WebhookGateway(LilyaPath, BaseInterceptorMixin, GatewayUtil):
         "permissions",
         "security",
         "tags",
+        "before_request",
+        "after_request",
     )
 
     def __init__(
         self,
         *,
         handler: Annotated[
-            Union["WebhookHandler", View],
+            Union["WebhookHandler", View, Type[View], Type["WebhookHandler"]],
             Doc(
                 """
                 An instance of [handler](https://esmerald.dev/routing/webhooks/#handlers).
@@ -621,6 +885,28 @@ class WebhookGateway(LilyaPath, BaseInterceptorMixin, GatewayUtil):
                 """
             ),
         ] = None,
+        before_request: Annotated[
+            Union[Sequence[Callable[[], Any]], None],
+            Doc(
+                """
+                A `list` of events that are trigger after the application
+                processes the request.
+
+                Read more about the [events](https://lilya.dev/lifespan/).
+                """
+            ),
+        ] = None,
+        after_request: Annotated[
+            Union[Sequence[Callable[[], Any]], None],
+            Doc(
+                """
+                A `list` of events that are trigger after the application
+                processes the request.
+
+                Read more about the [events](https://lilya.dev/lifespan/).
+                """
+            ),
+        ] = None,
         tags: Annotated[
             Optional[Sequence[str]],
             Doc(
@@ -644,14 +930,13 @@ class WebhookGateway(LilyaPath, BaseInterceptorMixin, GatewayUtil):
 
         if not name:
             if not isinstance(handler, View):
-                name = clean_string(handler.fn.__name__)
+                name = clean_string(handler.fn.__name__)  # type: ignore
             else:
                 name = clean_string(handler.__class__.__name__)
 
         self.handler = cast("Callable", handler)
         self.include_in_schema = include_in_schema
 
-        self._interceptors: Union[List["Interceptor"], "VoidType"] = Void
         self.name = name
         self.dependencies: Any = {}
         self.interceptors: Sequence["Interceptor"] = []
@@ -664,6 +949,8 @@ class WebhookGateway(LilyaPath, BaseInterceptorMixin, GatewayUtil):
         self.deprecated = deprecated
         self.parent = parent
         self.security = security
+        self.before_request = before_request
+        self.after_request = after_request
         self.tags = tags or []
         (handler.path_regex, handler.path_format, handler.param_convertors, _) = compile_path(
             self.path
@@ -674,5 +961,6 @@ class WebhookGateway(LilyaPath, BaseInterceptorMixin, GatewayUtil):
 
             if not handler.operation_id:
                 handler.operation_id = self.generate_operation_id(
-                    name=self.name, handler=self.handler  # type: ignore
+                    name=self.name,
+                    handler=self.handler,  # type: ignore
                 )
